/*-
 * Copyright (c) 2008 Andrey V. Surkov <nsand@sura.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: my_types.h,v 0.52 2008/05/23 07:54:36 nsand Exp $
 */


#include <curses.h>
#include <panel.h>
#include <net/ethernet.h>
#include <signal.h> 
#include <netinet/in.h>
#include <netinet/in_systm.h>

#ifndef _SKYFISH_TYPE_
#define	_SKYFISH_TYPE_

#if !defined(NCURSES_VERSION)
#error "Not found library NCURSES please install it"
#endif

static char VERSION[] = "0.83";

#define	LENTMP 320
#define	LENPATH 300
#define TIME_CHECK 1
#define SRCDSTSIZE 300		/*for src dst path name*/
#define TYPE_HTTP 0x50545448
#define READBUFSIZE 0xffff + ETHSIZE
#define MAXIFACE 15
#define DBTSIZE 100		/* buffer size filterdbt */
#define ETHSIZE sizeof(struct ether_header)
#define IP_H  sizeof(struct ip)
#define TCP_H   sizeof(struct tcphdr)
#define ClearTMP() { memset(tmp,0,LENTMP); }

/* storage all known type and size for filtering */
struct filter {
    u_int64_t min;
    u_int64_t max;
    char type[32];
};

/* for filtering text in HTTP header */
struct filterdbt {
    char search [100];
    char type[32];
};

/* for filtering in file (binary mode) */
struct filterdbb {
    u_char search [50];
    char type[32];
    u_long len;
};


/* stream structure */
struct pks {
    u_int64_t X;				// 64 bit length file ii Content-Range
    u_int64_t len;			// 64 bit length file
    u_int64_t lencur;			// 64 bit summ of bytes    
    u_long dst;				// 32 bit IP
    u_long src;				// 32 bit IP
    u_short port_d;			// 16 bit TCP
    u_short port_s;			// 16 bit TCP
    u_long ack;				// 32 bit TCP
    u_long http_offset;			// offset for begin file
    u_long start_count;			/* 32 bit TCP = ack 8 */
    u_long refer;			/* refer to main stream if present */
    u_long numfx;			/* count [1-3][5-7] */
    time_t atime;			/* last acces time to file */
    time_t ctime;			/* time to create file */
    time_t deltatime;			/* last acces in file in seconds */
    u_char begin;			/* key is 0- not begin or 200 - begin file */
    char http[3001];			/* fo storage header HTTP */
    u_long  httpf;			/* key for all data in http */
    char name[250];
    char findname[250];			/* if name found in HTTP */
    char type[32];			/* type file */
#define MASSIZE 250
    u_int64_t mas[MASSIZE];  		/* buffer for define fullness file size*/
    char etag[64];			/* Etag in HTTP protocol */
    int fd;
    int fordel;				/* mark for delete */
};

struct crossxy {
	u_int64_t x;
	u_int64_t y;
	int data;
};


#define SIZEOF64 sizeof(u_int64_t)
#define PKS sizeof(struct pks)

#endif 
