/*-
 * Copyright (c) 2008 Andrey V. Surkov <nsand@sura.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: parse_http.c,v 0.52 2008/05/23 07:54:36 nsand Exp $
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/ip_var.h>
#include <netinet/tcp.h>
#include <netinet/tcp_var.h>

#include <ncurses.h>
#include <panel.h>

#include "parse_http.h"
#include "skyfish.h"


extern u_long MAXTHREADS;
extern struct pks *db;
extern char tmp[];


int 
get_HTTP(const char *str)
{
    u_long res = 0;
    u_long *http = (u_long *)(str);
    if (*http == TYPE_HTTP) {
    	sscanf((str + 9),"%lu",&res);
    } else res = 0;
 return res;
}


/*
* parse packet by HTTP head
*/
int 
parse_TCP(const char *packet)
{
    struct ip *ip  	= (struct ip *) (packet + ETHSIZE);
    struct tcphdr *tcp	= (struct tcphdr *)(packet +  (ip->ip_hl<<2) + ETHSIZE);
    const char *data_tcp= (const char *) (packet + (ip->ip_hl<<2) + (tcp->th_off<<2) + ETHSIZE);
    const char *data 	= (const char *) (packet + ETHSIZE);
    u_long lendata = ip->ip_len - (ip->ip_hl<<2) - (tcp->th_off<<2);
    if (lendata == 0) return 1;
    u_long thseq = ntohl(tcp->th_seq);
    u_long numpks = MAXTHREADS;
    int type_http = get_HTTP(data_tcp);
    
    if (type_http >= 300 || type_http == 204)	return 1;
    if (type_http > 0 && type_http < 200) 	return 1;
    if (type_http == 0)		numpks = FindPKSDB(data); 
    if (type_http >= 200)	numpks =  AddPKSDB(data);
    
    if (numpks >= MAXTHREADS) return 1;

    if (db[numpks].httpf == 0) BuildHttp(data_tcp,lendata,numpks);  //   HTTP  
    if (db[numpks].httpf == 1) WriteData(data_tcp,numpks,lendata,thseq);

return 0;
}


void
BuildHttp(const char* data_tcp,u_long lendata,u_long numpks)
{
    u_long len = GetLenHTTP(data_tcp);
    u_long curlen = strlen(db[numpks].http);
    if ((len + curlen) >= 3000) { len = 3000 - curlen;}
    if ((curlen + lendata) >= 3000) { lendata = 3000 - curlen;}
    if (len != 0) { 
	strncat(db[numpks].http,data_tcp,len);
	db[numpks].http_offset += len; 
	db[numpks].httpf = 1;
	FillPKS(numpks);
	//if (db[numpks].start_count >600 && db[numpks].start_count < 650) Logger("Warning! start == 626\n");
	}
    if (len == 0) {
	strncat(db[numpks].http,data_tcp,lendata);
	db[numpks].http_offset += lendata;
	}
}


int
GetLenHTTP(const char* data)
{
    int offset=0;
    char *l;
    char CRLF[]={13,10,13,10,0};
    char CF[]={10,10,0};
    
	l = strstr(data,CRLF);
	if (l != NULL) {
	    offset = l - data;
	    return (offset + 4);
	}
	l = strstr(data,CF);	     
	if (l != NULL) {
	    offset = l - data;
	    return (offset + 2);
	} 
return 0;
}


//    
int
SearchFilename(u_long refer)
{
    u_long i;
    char *pos;
    if ((pos = strcasestr(db[refer].http,"filename="))  != NULL) {
	    pos+=9;
		for (i=0; i<249; i++)
		{
		    if (*pos != '"') {
		    db[refer].findname[i] = *pos; } else i--;
		    pos++;
		    if (*pos == 10 || *pos == 13 || *pos == ';' || *pos == '?' || *pos == '&') return 1;
    		}
    }
return 0;
}


int
SearchEtag(u_long refer)
{
    u_long i;
    char *pos;
    if ((pos = strcasestr(db[refer].http,"Etag:"))  != NULL) {
	    pos+=6;
		for (i=0; i<63; i++) 
		{
		    if (*pos != '"') {
		    db[refer].etag[i] = *pos; } else i--;
		    pos++;
		    if (*pos == 10 || *pos == 13 || *pos == ';' || *pos == '?' || *pos == '&') return 1;		    
		}
    }
return 0;
}
