/*-
 * Copyright (c) 2008 Andrey V. Surkov <nsand@sura.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	 $Id: sky_node.c,v 0.52 2008/05/23 07:54:36 nsand Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>

#include <sys/time.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/linker.h>
#include <sys/types.h>

#include <netgraph.h>

#include "sky_node.h"
#include "my_types.h"

int	csock, dsock;
char	name[NG_NODESIZ];

u_long numface;
extern char ifaces[MAXIFACE][100];

int
initnode(void)
{
    int res;

    res = kldload("netgraph");
    if (res != 0 && errno != EEXIST)	{ printf("error: netgraph %s\n", strerror(errno)); return 1;}
    res = kldload("ng_ether");
    if (res != 0 && errno != EEXIST)	{ printf("error: ng_ether %s\n", strerror(errno)); return 1;}
    res = kldload("ng_socket");
    if (res != 0 && errno != EEXIST)	{ printf("error: ng_socket %s\n", strerror(errno)); return 1;}
    res = kldload("ng_tee");
    if (res != 0 && errno != EEXIST)	{ printf("error: ng_tee %s\n", strerror(errno)); return 1;}
    res = kldload("ng_skyfish");
    if (res != 0 && errno != EEXIST)	{ printf("error: ng_skyfish %s\n", strerror(errno)); return 1;}
	snprintf(name, sizeof(name), "skyfish%d", getpid());
    	if (NgMkSockNode(name, &csock, &dsock) < 0)
		{printf("can't create socket to netgraph\n"); return 1;}
return 0;
}


int 
addnode(char *iface)
{
	char *pathall;
	pathall = (char *)calloc(100,1);
	if (pathall == NULL) { printf("calloc no mem for pathall. error\n");exit(1);}
	sprintf(pathall,"%s:",iface);

	struct ngm_mkpeer mkp;
		snprintf(mkp.type, sizeof(mkp.type), "tee");
		snprintf(mkp.ourhook, sizeof(mkp.ourhook), "lower");
		snprintf(mkp.peerhook, sizeof(mkp.peerhook), "right");

	if (NgSendMsg(csock, pathall, NGM_GENERIC_COOKIE,
	    NGM_MKPEER, &mkp, sizeof(mkp)) < 0) {
		printf("Error mkpeer tee lower right\n");
		return 1;
	}
	
	struct ngm_connect con;
		snprintf(con.path, sizeof(con.path), "lower");
		snprintf(con.ourhook, sizeof(con.ourhook), "upper");
		snprintf(con.peerhook, sizeof(con.peerhook), "left");

	if (NgSendMsg(csock, pathall, NGM_GENERIC_COOKIE,
	    NGM_CONNECT, &con, sizeof(con)) < 0) {
		printf("Error connect lower upper left\n");
		return (1);
	} 

	struct ngm_name name;
	sprintf(pathall,"%s:lower",iface);
	sprintf(name.name,"%s_acct_tee",iface);

	if (NgSendMsg(csock, pathall, NGM_GENERIC_COOKIE,
	    NGM_NAME, &name, sizeof(name)) < 0) {
		printf("Error name lower acct_tee\n");
		return (1);
	}
	
	sprintf(pathall,"%s_acct_tee:",iface);
	bzero(&mkp,sizeof(struct ngm_mkpeer));
		snprintf(mkp.type, sizeof(mkp.type), "skyfish");
		snprintf(mkp.ourhook, sizeof(mkp.ourhook), "right2left");
		snprintf(mkp.peerhook, sizeof(mkp.peerhook), "%s_in",iface);

	if (NgSendMsg(csock, pathall, NGM_GENERIC_COOKIE,
	    NGM_MKPEER, &mkp, sizeof(mkp)) < 0) {
		printf("Error mkpeer actt_tee skyfish right2left _in\n");
		return 1;
	}
	
	bzero(&name,sizeof(name));
	sprintf(pathall,"%s_acct_tee:right2left",iface);
	sprintf(name.name,"%s_ip_acct",iface);

	if (NgSendMsg(csock, pathall, NGM_GENERIC_COOKIE,
	    NGM_NAME, &name, sizeof(name)) < 0) {
		printf("Error name _acct_tee:right2left _ip_acct\n");
		return (1);
	}

	bzero(pathall,100);
	sprintf(pathall,"%s_acct_tee:",iface);
	bzero(&con,sizeof(con));
		snprintf(con.path, sizeof(con.path), "%s_ip_acct:",iface);
		snprintf(con.ourhook, sizeof(con.ourhook), "left2right");
		snprintf(con.peerhook, sizeof(con.peerhook), "%s_out",iface);

	if (NgSendMsg(csock, pathall, NGM_GENERIC_COOKIE,
	    NGM_CONNECT, &con, sizeof(con)) < 0) {
		printf("Error connect _acct_tee _ip_acct left2right _out\n");
		return (1);
	}

 free(pathall);
return 0;	
}



void
CloseSocket(void)
{
	if (NgSendMsg(csock, ".", NGM_GENERIC_COOKIE,
	    NGM_SHUTDOWN, NULL, 0) < 0) {
		printf("error shutdown skyfish\n");
	}
}



int 
shutnode(char *iface)
{
	char *pathall;
	pathall = (char *)calloc(100,1);
	if (pathall == NULL) { printf("calloc pathall error\n");exit(1);}
	
	sprintf(pathall,"%s_acct_tee:",iface);
	if (NgSendMsg(csock, pathall, NGM_GENERIC_COOKIE,
	    NGM_SHUTDOWN, NULL, 0) < 0) {
	}
	
	bzero(pathall,100);
	sprintf(pathall,"%s:",iface);

	if (NgSendMsg(csock, pathall, NGM_GENERIC_COOKIE,
	    NGM_SHUTDOWN, NULL, 0) < 0) {
	}
 free(pathall);
return 0;
}



int 
ShutNodes(void)
{
    int ii;
    for (ii=0;ii<numface;ii++) {
	if (shutnode(ifaces[ii]) != 0) return 1;
    }
return 0;
}



int
AddNodes(void)
{
    int ii;
    for (ii=0;ii<numface;ii++) {
	if (addnode(ifaces[ii]) != 0)  return 1;
    }
return 0;
}


u_long
Getnumface(void)
{
    return numface;
}


void
Addnumface(void)
{
    numface++;
}
